@@ -148,6 +148,9 @@ item *do_item_alloc(char *key, const size_t nkey, const unsigned int flags,
     uint8_t nsuffix;
     item *it = NULL;
     char suffix[40];
+    if (nbytes < 2 || nkey < 0)
+        return 0;
+
     size_t ntotal = item_make_header(nkey + 1, flags, nbytes, suffix, &nsuffix);
     if (settings.use_cas) {
         ntotal += sizeof(uint64_t);
@@ -303,7 +306,7 @@ static void do_item_link_q(item *it) { /* item is the new head */
     *head = it;
     if (*tail == 0) *tail = it;
     sizes[it->slabs_clsid]++;
-    sizes_bytes[it->slabs_clsid] += it->nbytes;
+    sizes_bytes[it->slabs_clsid] += ITEM_ntotal(it);
     return;
 }
 
@@ -332,7 +335,7 @@ static void do_item_unlink_q(item *it) {
     if (it->next) it->next->prev = it->prev;
     if (it->prev) it->prev->next = it->next;
     sizes[it->slabs_clsid]--;
-    sizes_bytes[it->slabs_clsid] -= it->nbytes;
+    sizes_bytes[it->slabs_clsid] -= ITEM_ntotal(it);
     return;
 }
 
@@ -484,10 +487,10 @@ char *item_cachedump(const unsigned int slabs_clsid, const unsigned int limit, u
         /* Copy the key since it may not be null-terminated in the struct */
         strncpy(key_temp, ITEM_key(it), it->nkey);
         key_temp[it->nkey] = 0x00; /* terminate */
-        len = snprintf(temp, sizeof(temp), "ITEM %s [%d b; %lu s]\r\n",
+        len = snprintf(temp, sizeof(temp), "ITEM %s [%d b; %llu s]\r\n",
                        key_temp, it->nbytes - 2,
                        it->exptime == 0 ? 0 :
-                       (unsigned long)it->exptime + process_started);
+                       (unsigned long long)it->exptime + process_started);
         if (bufcurr + len + 6 > memlimit)  /* 6 is END\r\n\0 */
             break;
         memcpy(buffer + bufcurr, temp, len);
@@ -589,6 +592,8 @@ void item_stats(ADD_STAT add_stats, void *c) {
             lru_size_map[x] = sizes[i];
             if (lru_type_map[x] == COLD_LRU && tails[i] != NULL)
                 age = current_time - tails[i]->time;
+            if (lru_type_map[x] == COLD_LRU)
+                totals.evicted_time = itemstats[i].evicted_time;
             pthread_mutex_unlock(&lru_locks[i]);
         }
         if (size == 0)
